# -*- coding: utf-8 -*-

import typing as T
import json
import dataclasses

from .paths import dir_data


@dataclasses.dataclass
class BaseModel:
    @classmethod
    def from_dict(cls, data: T.Dict[str, T.Any]):
        new_data = {}
        for field in dataclasses.fields(cls):
            if data.get(field.name) is not None:
                new_data[field.name] = data[field.name]
        return cls(**new_data)

    def to_dict(self) -> T.Dict[str, T.Any]:
        return dataclasses.asdict(self)


@dataclasses.dataclass
class SubService(BaseModel): # pragma: no cover
    """
    Refer to a sub service of a main service. For example, AWS S3 bucket
    is a sub service of AWS S3.

    :param id: unique id, for example, "buckets" is for AWS S3 buckets
    :param name: e.g. "Buckets"
    :param url: the href part of the url, for example, "the full url" is
        https://console.aws.amazon.com/s3/buckets?region=us-east-1
        then the "url" is "/s3/buckets"
    :param weight: the sorting weight in the searching
    :param short_name: e.g. "buckets"
    :param search_terms: additional search terms for ngram search
    """

    id: str = dataclasses.field()
    name: str = dataclasses.field()
    url: str = dataclasses.field()
    weight: int = dataclasses.field(default=1)  # 1 - 100
    short_name: T.Optional[str] = dataclasses.field(default=None)
    search_terms: T.List[str] = dataclasses.field(default_factory=list)


@dataclasses.dataclass
class MainService(BaseModel): # pragma: no cover
    """
    Refer to a main service. For example, AWS S3 is a main service.

    :param id: unique id, for example, "s3" is for AWS S3
    :param name: e.g. "Amazon Simple Storage Service"
    :param url: the href part of the url, for example, "the full url" is
        https://console.aws.amazon.com/s3/buckets?region=us-east-1
        then the "url" is "/s3/buckets"
    :param regional: see https://aws.amazon.com/about-aws/global-infrastructure/regional-product-services/
    :param weight: the sorting weight in the searching
    :param short_name: e.g. "S3"
    :param description: a human friendly description
    :param search_terms: additional search terms for ngram search
    :param sub_services: a list of sub services
    """

    id: str = dataclasses.field()
    name: str = dataclasses.field()
    url: str = dataclasses.field()
    regional: bool = dataclasses.field(default=True)
    weight: int = dataclasses.field(default=1)  # 1 - 1000
    short_name: T.Optional[str] = dataclasses.field(default=None)
    description: T.Optional[str] = dataclasses.field(default=None)
    search_terms: T.List[str] = dataclasses.field(default_factory=list)
    sub_services: T.List[SubService] = dataclasses.field(default_factory=list)


def load_data() -> T.List[MainService]:
    main_services = list()
    for path_json in dir_data.select_by_ext(".json"):
        data = json.loads(path_json.read_text())
        data["sub_services"] = [SubService.from_dict(dct) for dct in data["sub_services"]]
        main_service = MainService.from_dict(data)
        main_services.append(main_service)
    return main_services



# -*- coding: utf-8 -*-

"""
This module provides a base class for each whoosh search index.
"""

import dataclasses
from pathlib_mate import Path
from whoosh import fields
from whoosh.index import open_dir, create_in, FileIndex
from ..cache import dir_cache


@dataclasses.dataclass
class SearchIndex:
    """
    Base class for whoosh search index. It has to implement the following methods:

    - ``build_index()``:
    - ``search()``:
    """
    schema: fields.SchemaClass = dataclasses.field()
    dir_index: Path = dataclasses.field()

    def __post_init__(self):
        self.dir_index = Path(self.dir_index)

    def get_index(self) -> FileIndex:
        if self.dir_index.exists():
            idx = open_dir(self.dir_index.abspath)
        else:
            self.dir_index.mkdir(parents=True, exist_ok=True)
            idx = create_in(dirname=self.dir_index.abspath, schema=self.schema)
        return idx

    def clear_cache(self):
        dir_cache.remove_if_exists()



# -*- coding: utf-8 -*-

"""
AWS Console URL full text search utility.
"""


from ._version import __version__

__short_description__ = "AWS Console URL full text search utility."
__license__ = "MIT"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"



# -*- coding: utf-8 -*-

import typing as T
import os
import dataclasses
from pathlib_mate import Path
from whoosh import fields, qparser, query, sorting

from ..model import BaseModel, MainService, load_data
from ..paths import dir_sub_service_index
from ..cache import cache
from ..constants import CACHE_EXPIRE
from .base import SearchIndex


class SubServiceSchema(fields.SchemaClass):
    id = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
        sortable=True,
    )
    id_kw = fields.KEYWORD(stored=False)
    name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
    )
    short_name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
    )
    url = fields.STORED()
    weight = fields.NUMERIC(
        sortable=True,
        stored=True,
    )
    search_terms = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=False,
        field_boost=2.0,
    )
    main_svc_id = fields.KEYWORD(stored=True)


sub_service_schema = SubServiceSchema()


@dataclasses.dataclass
class SubServiceDocument(BaseModel):
    main_svc_id: str = dataclasses.field()
    id: str = dataclasses.field()
    name: str = dataclasses.field()
    url: str = dataclasses.field()
    weight: int = dataclasses.field()
    short_name: T.Optional[str] = dataclasses.field(default=None)

    @property
    def title(self) -> str:
        return f"{self.main_svc_id}-{self.id}"

    @property
    def subtitle(self) -> str:
        if self.short_name:
            return f"{self.name} ({self.short_name})"
        else:
            return self.name


@dataclasses.dataclass
class SubServiceIndex(SearchIndex):
    @classmethod
    def new(
        cls,
        dir_index: Path = dir_sub_service_index,
    ) -> "SubServiceIndex":
        return SubServiceIndex(
            schema=sub_service_schema,
            dir_index=dir_index,
        )

    def _build_index(
        self,
        main_services: T.Optional[T.List[MainService]] = None,
        multi_thread: bool = False,
    ):
        """
        Build Whoosh Index, add document.
        """
        if main_services is None:
            main_services = load_data()

        idx = self.get_index()
        if multi_thread:  # pragma: no cover
            writer = idx.writer(procs=os.cpu_count())
        else:
            writer = idx.writer()

        for main_service in main_services:
            for sub_service in main_service.sub_services:
                document = {
                    "id": sub_service.id,
                    "id_kw": sub_service.id,
                    "name": sub_service.name,
                    "short_name": sub_service.short_name,
                    "url": sub_service.url,
                    "search_terms": " ".join(sub_service.search_terms),
                    "weight": sub_service.weight,
                }
                new_document = {k: v for k, v in document.items() if bool(v)}
                new_document["main_svc_id"] = main_service.id
                writer.add_document(**new_document)
        writer.commit()

    def build_index(
        self,
        main_services: T.Optional[T.List[MainService]] = None,
        multi_thread: bool = False,
        rebuild: bool = False,
    ):
        if rebuild:
            self.dir_index.remove_if_exists()
        self._build_index(
            main_services=main_services,
            multi_thread=multi_thread,
        )

    @cache.memoize(expire=CACHE_EXPIRE)
    def get_by_id(
        self,
        main_svc_id: str,
        sub_svc_id: str,
    ) -> T.Optional[SubServiceSchema]:
        q = query.And(
            [
                query.Term("main_svc_id", main_svc_id),
                query.Term("id_kw", sub_svc_id),
            ]
        )
        idx = self.get_index()
        with idx.searcher() as searcher:
            doc_list = [hit.fields() for hit in searcher.search(q, limit=1)]
        try:
            return SubServiceDocument.from_dict(doc_list[0])
        except IndexError:
            return None

    @cache.memoize(expire=CACHE_EXPIRE)
    def top_k(
        self,
        main_svc_id: str,
        k: int = 20,
    ) -> T.List[SubServiceDocument]:
        q = query.Term("main_svc_id", main_svc_id)
        idx = self.get_index()
        multi_facet = sorting.MultiFacet()
        multi_facet.add_field("weight", reverse=True)
        multi_facet.add_field("id")
        doc_list = list()
        with idx.searcher() as searcher:
            for hit in searcher.search(q, sortedby=multi_facet, limit=k):
                doc_list.append(SubServiceDocument.from_dict(hit.fields()))
        return doc_list

    @cache.memoize(expire=CACHE_EXPIRE)
    def search(
        self,
        main_svc_id: str,
        query_str: str,
        limit: int = 20,
    ) -> T.List[SubServiceDocument]:
        q = query.And(
            [
                query.Term("main_svc_id", main_svc_id),
                qparser.MultifieldParser(
                    [
                        "id",
                        "name",
                        "short_name",
                        "search_terms",
                    ],
                    schema=self.schema,
                ).parse(query_str),
            ]
        )
        index = self.get_index()
        multi_facet = sorting.MultiFacet()
        multi_facet.add_field("weight", reverse=True)
        multi_facet.add_field("id")
        doc_list = list()
        with index.searcher() as searcher:
            for hit in searcher.search(q, sortedby=multi_facet, limit=limit):
                doc_list.append(SubServiceDocument.from_dict(hit.fields()))

        # if the "id" starts with the query str, prioritize it
        doc_list_1 = list()
        doc_list_2 = list()
        for doc in doc_list:
            if doc.id.startswith(query_str):
                doc_list_1.append(doc)
            else:
                doc_list_2.append(doc)
        doc_list_1.extend(doc_list_2)
        return doc_list_1



# -*- coding: utf-8 -*-

import typing as T
from aws_console_url_search.tests import run_cov_test
from aws_console_url_search.index.sub_svc import (
    SubServiceIndex,
)
from aws_console_url_search.paths import dir_tests


class TestSubServiceIndex:
    index: T.Optional[SubServiceIndex] = None

    @classmethod
    def setup_class(cls):
        cls.index = SubServiceIndex.new(
            dir_index=dir_tests.joinpath("tmp", "sub_service_index")
        )
        cls.index.build_index(rebuild=True)
        cls.index.clear_cache()

    def test_search_one(self):
        test_cases = [
            ("ec2", "securitygroups", "securitygroups"),
            ("ec2", "invalid", None),
        ]
        for main_svc_id, sub_svc_id, expected_sub_svc_id in test_cases:
            if expected_sub_svc_id is None:
                assert self.index.get_by_id(main_svc_id, sub_svc_id) is None
            else:
                assert (
                    self.index.get_by_id(main_svc_id, sub_svc_id).id
                    == expected_sub_svc_id
                )

    def test_top_k(self):
        for _ in range(10):
            ids = [doc.id for doc in self.index.top_k("ec2")]
            assert ids[:4] == [
                "instances",
                "securitygroups",
                "amis",
                "volumes",
            ]

    def test_search(self):
        test_cases = [
            ("ec2", "inst", "instances"),
            ("ec2", "instance", "instances"),
            ("ec2", "security group", "securitygroups"),
            ("ec2", "securitygroup", "securitygroups"),
            ("ec2", "sg", "securitygroups"),
            ("ec2", "ebs", "volumes"),
            ("ec2", "volume", "volumes"),
            ("ec2", "elastic block storage", "volumes"),
            ("iam", "role", "roles"),
            ("iam", "policy", "policies"),
        ]
        for main_svc_id, query_str, sub_svc_id in test_cases:
            assert self.index.search(main_svc_id, query_str)[0].id == sub_svc_id

    def test_title_and_subtitle(self):
        for doc in self.index.top_k("ec2"):
            _ = doc.title
            _ = doc.subtitle


if __name__ == "__main__":
    run_cov_test(__file__, "aws_console_url_search.index.sub_svc", preview=False)



# -*- coding: utf-8 -*-

import typing as T
import dataclasses
from aws_console_url_search.model import (
    BaseModel,
    load_data,
)


@dataclasses.dataclass
class Model(BaseModel):
    id: str = dataclasses.field()
    name: T.Optional[str] = dataclasses.field(default=None)
    flag: bool = dataclasses.field(default=True)


class TestBaseModel:
    def test(self):
        data = {"id": "id-1", "flag": None, "undefined": 123}
        model = Model.from_dict(data)
        assert model.id == "id-1"
        assert model.name is None
        assert model.flag is True

        assert model.to_dict() == {"id": "id-1", "name": None, "flag": True}


def test_load_data():
    main_services = load_data()

    _ = main_services


if __name__ == "__main__":
    from aws_console_url_search.tests import run_cov_test

    run_cov_test(__file__, "aws_console_url_search.model", preview=False)



# -*- coding: utf-8 -*-

from .any_svc import (
    AnyServiceDocument,
    AnyServiceIndex,
)
from .main_svc import (
    MainServiceDocument,
    MainServiceIndex,
)
from .sub_svc import (
    SubServiceDocument,
    SubServiceIndex,
)


# -*- coding: utf-8 -*-

"""
A wrapper around the whoosh index object.
"""

import typing as T
import os
import dataclasses
from pathlib_mate import Path
from whoosh import fields, qparser, query, sorting

from ..model import BaseModel, MainService, load_data
from ..paths import dir_sub_service_index
from ..cache import cache
from ..constants import CACHE_EXPIRE
from .base import SearchIndex


class AnyServiceSchema(fields.SchemaClass):
    main_svc_id = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
        sortable=True,
        field_boost=3.0,
    )
    main_svc_id_kw = fields.KEYWORD(stored=False)
    main_svc_name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
    )
    main_svc_short_name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
        field_boost=3.0,
    )
    main_svc_description = fields.STORED()
    main_svc_url = fields.STORED()
    main_svc_regional = fields.BOOLEAN(stored=True)
    main_svc_weight = fields.NUMERIC(
        sortable=True,
        stored=True,
    )
    main_svc_search_terms = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=False,
        field_boost=2.0,
    )
    main_svc_has_sub_svc = fields.STORED()

    sub_svc_id = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
        sortable=True,
    )
    sub_svc_id_kw = fields.KEYWORD(stored=False)
    sub_svc_name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
    )
    sub_svc_short_name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
    )
    sub_svc_url = fields.STORED()
    sub_svc_weight = fields.NUMERIC(
        sortable=True,
        stored=True,
    )
    sub_svc_search_terms = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=False,
        field_boost=2.0,
    )


any_service_schema = AnyServiceSchema()


@dataclasses.dataclass
class AnyServiceDocument(BaseModel):
    main_svc_id: str = dataclasses.field()
    main_svc_name: str = dataclasses.field()
    main_svc_url: str = dataclasses.field()
    main_svc_regional: bool = dataclasses.field()
    main_svc_weight: int = dataclasses.field()
    main_svc_has_sub_svc: bool = dataclasses.field()
    main_svc_short_name: T.Optional[str] = dataclasses.field(default=None)
    main_svc_description: T.Optional[str] = dataclasses.field(default=None)

    sub_svc_id: T.Optional[str] = dataclasses.field(default=None)
    sub_svc_name: T.Optional[str] = dataclasses.field(default=None)
    sub_svc_url: T.Optional[str] = dataclasses.field(default=None)
    sub_svc_weight: T.Optional[int] = dataclasses.field(default=None)
    sub_svc_short_name: T.Optional[str] = dataclasses.field(default=None)

    @property
    def title(self) -> str:
        if self.sub_svc_id:
            return f"{self.main_svc_id}-{self.sub_svc_id}"
        else:
            return self.main_svc_id

    @property
    def subtitle(self) -> str:
        if self.sub_svc_id:
            if self.sub_svc_short_name:
                return f"{self.sub_svc_name} ({self.sub_svc_short_name})"
            else:
                return self.sub_svc_name
        else:
            if self.main_svc_description:
                return f"{self.main_svc_name} - {self.main_svc_description}"
            else:
                return self.main_svc_name


@dataclasses.dataclass
class AnyServiceIndex(SearchIndex):
    @classmethod
    def new(
        cls,
        dir_index: Path = dir_sub_service_index,
    ) -> "AnyServiceIndex":
        return AnyServiceIndex(
            schema=any_service_schema,
            dir_index=dir_index,
        )

    def _build_index(
        self,
        main_services: T.Optional[T.List[MainService]] = None,
        multi_thread: bool = False,
    ):
        """
        Build Whoosh Index, add document.
        """
        if main_services is None:
            main_services = load_data()

        idx = self.get_index()
        if multi_thread: # pragma: no cover
            writer = idx.writer(procs=os.cpu_count())
        else:
            writer = idx.writer()

        for main_service in main_services:
            document = {
                "main_svc_id": main_service.id,
                "main_svc_id_kw": main_service.id,
                "main_svc_name": main_service.name,
                "main_svc_short_name": main_service.short_name,
                "main_svc_description": main_service.description,
                "main_svc_url": main_service.url,
                "main_svc_search_terms": " ".join(main_service.search_terms),
                "main_svc_weight": main_service.weight,
            }
            new_document = {k: v for k, v in document.items() if bool(v)}
            new_document["main_svc_has_sub_svc"] = bool(main_service.sub_services)
            new_document["main_svc_regional"] = main_service.regional
            writer.add_document(**new_document)
            for sub_service in main_service.sub_services:
                document = {
                    "main_svc_id": main_service.id,
                    "main_svc_id_kw": main_service.id,
                    "main_svc_name": main_service.name,
                    "main_svc_short_name": main_service.short_name,
                    "main_svc_description": main_service.description,
                    "main_svc_url": main_service.url,
                    "main_svc_search_terms": " ".join(main_service.search_terms),
                    "main_svc_weight": main_service.weight,
                    "sub_svc_id": sub_service.id,
                    "sub_svc_id_kw": sub_service.id,
                    "sub_svc_name": sub_service.name,
                    "sub_svc_short_name": sub_service.short_name,
                    "sub_svc_url": sub_service.url,
                    "sub_svc_search_terms": " ".join(sub_service.search_terms),
                    "sub_svc_weight": sub_service.weight,
                }
                new_document = {k: v for k, v in document.items() if bool(v)}
                new_document["main_svc_regional"] = main_service.regional
                new_document["main_svc_has_sub_svc"] = bool(main_service.sub_services)
                writer.add_document(**new_document)
        writer.commit()

    def build_index(
        self,
        main_services: T.Optional[T.List[MainService]] = None,
        multi_thread: bool = False,
        rebuild: bool = False,
    ):
        if rebuild:
            self.dir_index.remove_if_exists()
        self._build_index(
            main_services=main_services,
            multi_thread=multi_thread,
        )

    @cache.memoize(expire=CACHE_EXPIRE)
    def search(
        self,
        query_str: str,
        limit: int = 20,
    ) -> T.List[AnyServiceDocument]:
        q = query.And(
            [
                qparser.MultifieldParser(
                    [
                        "main_svc_id",
                        "main_svc_name",
                        "main_svc_short_name",
                        "main_svc_search_terms",
                        "sub_svc_id",
                        "sub_svc_name",
                        "sub_svc_short_name",
                        "sub_svc_search_terms",
                    ],
                    schema=self.schema,
                ).parse(query_str),
            ]
        )
        index = self.get_index()
        multi_facet = sorting.MultiFacet()
        multi_facet.add_field("main_svc_weight", reverse=True)
        multi_facet.add_field("main_svc_id")
        multi_facet.add_field("sub_svc_weight", reverse=True)
        multi_facet.add_field("sub_svc_id")
        doc_list = list()
        with index.searcher() as searcher:
            for hit in searcher.search(q, sortedby=multi_facet, limit=limit):
                doc_list.append(AnyServiceDocument.from_dict(hit.fields()))

        return doc_list



# -*- coding: utf-8 -*-

from pathlib_mate import Path

_dir_here = Path.dir_here(__file__)

dir_project_root = _dir_here.parent
dir_data = _dir_here.joinpath("data")

# ------------------------------------------------------------------------------
# ${HOME}/.aws_console_url_search/ dir related
# ------------------------------------------------------------------------------
dir_home = Path.home()
dir_aws_console_url_search = dir_home.joinpath(".aws_console_url_search")
dir_cache = dir_aws_console_url_search.joinpath("cache")
dir_main_service_index = dir_aws_console_url_search.joinpath("main_service_index")
dir_sub_service_index = dir_aws_console_url_search.joinpath("sub_service_index")
dir_any_service_index = dir_aws_console_url_search.joinpath("any_service_index")

# ------------------------------------------------------------------------------
# test related
# ------------------------------------------------------------------------------
dir_tests = dir_project_root.joinpath("tests")



# -*- coding: utf-8 -*-

import typing as T
import os
import dataclasses
from pathlib_mate import Path
from whoosh import fields, qparser, query, sorting

from ..model import BaseModel, MainService, load_data
from ..paths import dir_main_service_index
from ..cache import cache
from ..constants import CACHE_EXPIRE
from .base import SearchIndex


class MainServiceSchema(fields.SchemaClass):
    id = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
        sortable=True,
        field_boost=3.0,
    )
    id_kw = fields.KEYWORD(stored=False)
    name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
    )
    short_name = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=True,
        field_boost=3.0,
    )
    description = fields.STORED()
    url = fields.STORED()
    regional = fields.BOOLEAN(stored=True)
    weight = fields.NUMERIC(
        sortable=True,
        stored=True,
    )
    search_terms = fields.NGRAM(
        minsize=2,
        maxsize=10,
        stored=False,
        field_boost=2.0,
    )
    has_sub_svc = fields.STORED()


main_service_schema = MainServiceSchema()


@dataclasses.dataclass
class MainServiceDocument(BaseModel):
    id: str = dataclasses.field()
    name: str = dataclasses.field()
    url: str = dataclasses.field()
    regional: bool = dataclasses.field()
    weight: int = dataclasses.field()
    has_sub_svc: bool = dataclasses.field()
    short_name: T.Optional[str] = dataclasses.field(default=None)
    description: T.Optional[str] = dataclasses.field(default=None)

    @property
    def title(self) -> str:
        return self.id

    @property
    def subtitle(self) -> str:
        if self.description:
            return f"{self.name} - {self.description}"
        else:
            return self.name


@dataclasses.dataclass
class MainServiceIndex(SearchIndex):
    @classmethod
    def new(
        cls,
        dir_index: Path = dir_main_service_index,
    ) -> "MainServiceIndex":
        return MainServiceIndex(
            schema=main_service_schema,
            dir_index=dir_index,
        )

    def _build_index(
        self,
        main_services: T.Optional[T.List[MainService]] = None,
        multi_thread: bool = False,
    ):
        if main_services is None:
            main_services = load_data()

        idx = self.get_index()
        if multi_thread:  # pragma: no cover
            writer = idx.writer(procs=os.cpu_count())
        else:
            writer = idx.writer()

        for main_service in main_services:
            document = {
                "id": main_service.id,
                "id_kw": main_service.id,
                "name": main_service.name,
                "short_name": main_service.short_name,
                "description": main_service.description,
                "url": main_service.url,
                "search_terms": " ".join(main_service.search_terms),
                "weight": main_service.weight,
            }

            new_document = {k: v for k, v in document.items() if bool(v)}
            new_document["regional"] = main_service.regional
            new_document["has_sub_svc"] = bool(main_service.sub_services)
            writer.add_document(**new_document)
        writer.commit()

    def build_index(
        self,
        main_services: T.Optional[T.List[MainService]] = None,
        multi_thread: bool = False,
        rebuild: bool = False,
    ):
        if rebuild:  # pragma: no cover
            self.dir_index.remove_if_exists()
        self._build_index(
            main_services=main_services,
            multi_thread=multi_thread,
        )

    @cache.memoize(expire=CACHE_EXPIRE)
    def get_by_id(self, id: str) -> T.Optional[MainServiceDocument]:
        q = query.Term("id_kw", id)
        idx = self.get_index()
        with idx.searcher() as searcher:
            doc_list = [hit.fields() for hit in searcher.search(q, limit=1)]
        try:
            return MainServiceDocument.from_dict(doc_list[0])
        except IndexError:
            return None

    @cache.memoize(expire=CACHE_EXPIRE)
    def top_k(self, k: int = 20) -> T.List[MainServiceDocument]:
        q = query.Or([query.Term("regional", True), query.Term("regional", False)])
        idx = self.get_index()
        multi_facet = sorting.MultiFacet()
        multi_facet.add_field("weight", reverse=True)
        multi_facet.add_field("id")
        doc_list = list()
        with idx.searcher() as searcher:
            for hit in searcher.search(q, sortedby=multi_facet, limit=k):
                doc_list.append(MainServiceDocument.from_dict(hit.fields()))
        return doc_list

    @cache.memoize(expire=CACHE_EXPIRE)
    def search(
        self,
        query_str: str,
        limit: int = 20,
    ) -> T.List[MainServiceDocument]:
        q = qparser.MultifieldParser(
            [
                "id",
                "name",
                "short_name",
                "search_terms",
            ],
            schema=self.schema,
        ).parse(query_str)
        index = self.get_index()
        multi_facet = sorting.MultiFacet()
        multi_facet.add_field("weight", reverse=True)
        multi_facet.add_field("id")
        doc_list = list()
        with index.searcher() as searcher:
            for hit in searcher.search(q, sortedby=multi_facet, limit=limit):
                doc_list.append(MainServiceDocument.from_dict(hit.fields()))

        # if the "id" starts with the query str, prioritize it
        doc_list_1 = list()
        doc_list_2 = list()
        for doc in doc_list:
            if doc.id.startswith(query_str):
                doc_list_1.append(doc)
            else:
                doc_list_2.append(doc)
        doc_list_1.extend(doc_list_2)
        return doc_list_1



# -*- coding: utf-8 -*-

import typing as T
from aws_console_url_search.tests import run_cov_test
from aws_console_url_search.index.main_svc import (
    MainServiceIndex,
)
from aws_console_url_search.paths import dir_tests


class TestMainServiceIndex:
    index: T.Optional[MainServiceIndex] = None

    @classmethod
    def setup_class(cls):
        cls.index = MainServiceIndex.new(
            dir_index=dir_tests.joinpath("tmp", "main_service_index")
        )
        cls.index.build_index(rebuild=True)
        cls.index.clear_cache()

    def test_search_one(self):
        test_cases = [
            ("ec2", "ec2"),
            ("s3", "s3"),
            ("vpc", "vpc"),
            ("iam", "iam"),
            ("invalid", None),
        ]
        for query_str, id in test_cases:
            if id is None:
                assert self.index.get_by_id(query_str) is None
            else:
                assert self.index.get_by_id(query_str).id == id

    def test_top_k(self):
        for _ in range(10):
            ids = [doc.id for doc in self.index.top_k()]
            assert ids[:4] == [
                "iam",
                "s3",
                "ec2",
                "vpc",
            ]

    def test_search(self):
        test_cases = [
            ("ec2", "ec2"),
            ("elastic compute", "ec2"),
            ("oss", "aos"),
            ("opensearch", "aos"),
            ("open search", "aos"),
            ("sage", "sagemaker"),
            ("sagemaker", "sagemaker"),
            ("sage maker", "sagemaker"),
            ("machine learning", "sagemaker"),
            ("ml", "sagemaker"),
        ]
        for query_str, id in test_cases:
            assert self.index.search(query_str)[0].id == id

    def test_title_and_subtitle(self):
        for doc in self.index.top_k(k=100):
            _ = doc.title
            _ = doc.subtitle


if __name__ == "__main__":
    run_cov_test(__file__, "aws_console_url_search.index.main_svc", preview=False)



# -*- coding: utf-8 -*-

import typing as T
from aws_console_url_search.tests import run_cov_test
from aws_console_url_search.index.any_svc import (
    AnyServiceIndex,
)
from aws_console_url_search.paths import dir_tests


class TestAnyServiceIndex:
    index: T.Optional[AnyServiceIndex] = None

    @classmethod
    def setup_class(cls):
        cls.index = AnyServiceIndex.new(
            dir_index=dir_tests.joinpath("tmp", "any_service_index")
        )
        cls.index.build_index(rebuild=True)
        cls.index.clear_cache()

    def test_search(self):
        test_cases = [
            ("ec2 inst", "ec2", "instances"),
            ("ec2 sg", "ec2", "securitygroups"),
        ]
        for query_str, main_svc_id, sub_svc_id in test_cases:
            doc = self.index.search(query_str)[0]
            assert doc.main_svc_id == main_svc_id
            assert doc.sub_svc_id == sub_svc_id

    def test_title_and_subtitle(self):
        for doc in self.index.search("ec2"):
            _ = doc.title
            _ = doc.subtitle


if __name__ == "__main__":
    run_cov_test(__file__, "aws_console_url_search.index.any_svc", preview=False)
